@@ -7,7 +7,7 @@
 check_diagram <- function(d,ret){
 
   # error checks for a diagram d stored as a data frame, and conversion
-  if((is.list(d) && ((length(d) == 1 && all(names(d) %in% "diagram") && methods::is(d$diagram,"diagram")) || ((length(d) == 4 && all(names(d) %in% c("diagram","birthLocation","deathLocation","cycleLocation")) && methods::is(d$diagram,"diagram"))))) || (methods::is(d,"matrix") && methods::is(d,"array") & all(colnames(d) %in% c("dimension","birth","death"))))
+  if((is.list(d) && ((length(d) == 1 && all(names(d) %in% "diagram") && (methods::is(d$diagram,"diagram")) || methods::is(d$diagram,"data.frame")) || ((length(d) == 4 && all(names(d) %in% c("diagram","birthLocation","deathLocation","cycleLocation")) && methods::is(d$diagram,"diagram"))))) || (methods::is(d,"matrix") && methods::is(d,"array") & all(colnames(d) %in% c("dimension","birth","death"))))
   {
     # d is the output from a TDA/TDAstats calculation
     d <- diagram_to_df(d)
@@ -59,6 +59,11 @@ check_diagram <- function(d,ret){
     stop("Death values must always be at least as large as birth values.")
   }
   
+  # if(length(which(is.infinite(d[,3L]))) > 0)
+  # {
+  #   stop("Death values must be finite.")
+  # }
+  
   if(ret == T)
   {
     return(d) 
